@@ -11,7 +11,7 @@ set_wpp_regions <- function(start.year=1950, present.year=2005, wpp.year=2008, m
 	# set data and match with areas
 	########################################
 	tfr_data <- read.UNtfr(wpp.year=wpp.year, my.tfr.file=my.tfr.file, 
-								present.year=present.year, verbose=verbose)$tfr_data
+								present.year=present.year, verbose=verbose)$data
 	# not just countries, includes areas etc as well
 	# get region and area data
 	locations <- read.UNlocations(tfr_data, wpp.year=wpp.year, verbose=verbose)
@@ -91,10 +91,10 @@ do.read.un.file <- function(un.file.name, wpp.year, my.file=NULL, present.year=2
 read.UNtfr <- function(wpp.year, my.tfr.file=NULL, ...) {
 	un.file.name <- file.path(.find.package("bayesTFR"), "data", paste('UN', wpp.year, '.txt', sep=''))
 	result <- do.read.un.file(un.file.name, wpp.year, my.file=my.tfr.file, ...)
-	return(list(tfr_data=result$data, replaced=result$replaced, added=result$added))
+	return(list(data=result$data, replaced=result$replaced, added=result$added))
 }
 
-read.UNlocations <- function(tfr_data, wpp.year, package="bayesTFR", verbose=FALSE) {
+read.UNlocations <- function(data, wpp.year, package="bayesTFR", verbose=FALSE) {
 	loc.file.name <- file.path(.find.package(package), "data", 
 								paste('WPP', wpp.year, '_LOCATIONS', '.txt', sep=''))
 	cat('Reading file ', loc.file.name, '.\n')
@@ -102,16 +102,16 @@ read.UNlocations <- function(tfr_data, wpp.year, package="bayesTFR", verbose=FAL
 	
 	# this include some areas that are not in the tfr file
 	# get the include_code from this file to get the countries from the tfr file
-	nr_tfr_outcomes <- length(tfr_data[,1])
+	nr_tfr_outcomes <- length(data[,1])
 	include <- rep(NA, nr_tfr_outcomes)
 	prediction.only <- rep(NA, nr_tfr_outcomes)
 	n_loc_data <- length(loc_data[,1])
 
 	for (i in 1:nr_tfr_outcomes){
- 		loc_index <- (1:n_loc_data)[loc_data$country_code == tfr_data$country_code[i]]
+ 		loc_index <- (1:n_loc_data)[loc_data$country_code == data$country_code[i]]
  		if (length(loc_index)<=0)
- 			stop('Country ', tfr_data$country_code[i], ' not found in the file ', loc.file.name)
- 		incl.code <- if(tfr_data$include_code[i] >= 0) tfr_data$include_code[i] else loc_data$include_code[loc_index]
+ 			stop('Country ', data$country_code[i], ' not found in the file ', loc.file.name)
+ 		incl.code <- if(data$include_code[i] >= 0) data$include_code[i] else loc_data$include_code[loc_index]
  		include[i] <- incl.code == 2
  		prediction.only[i] <- incl.code == 1
 	}
@@ -163,23 +163,22 @@ get.TFRmatrix.and.regions <- function(tfr_data, ..., verbose=FALSE){
 	return(list(tfr_matrix=result$obs_matrix, tfr_matrix_all=result$obs_matrix_all, regions=result$regions))
 }
 
-set.wpp.extra <- function(meta, countries=NULL, my.tfr.file=NULL, verbose=FALSE) {
-	#'countries' is a vector of country or region codes 
-	tfrs <- read.UNtfr(wpp.year=meta$wpp.year, my.tfr.file=my.tfr.file, 
-							present.year=meta$present.year, verbose=verbose)
+.extra.matrix.regions <- function(data, countries, meta, package="bayesTFR", verbose=FALSE) {
+	tfrs <- data
 	country.codes.processed <- meta$regions$country_code
-	ncountries <- meta$regions$nr_countries
+	ncountries <- get.nr.countries(meta)
+	ncountries.est <- get.nr.countries.est(meta)
 	replaced.processed <- intersect(tfrs$replaced, country.codes.processed)
 	idx.replaced.processed <- is.element(country.codes.processed, replaced.processed)
-	replaced.processed.est <- country.codes.processed[idx.replaced.processed[1:meta$nr_countries_estimation]]
+	replaced.processed.est <- country.codes.processed[idx.replaced.processed[1:ncountries.est]]
 	
 	added.processed <- intersect(tfrs$added, country.codes.processed)
 	idx.added.processed <- is.element(country.codes.processed, added.processed)
-	added.processed.est <- country.codes.processed[idx.added.processed[1:meta$nr_countries_estimation]]
+	added.processed.est <- country.codes.processed[idx.added.processed[1:ncountries.est]]
 	
 	countries.processed <- intersect(countries, country.codes.processed)
 	idx.countries.processed <- is.element(country.codes.processed, countries.processed)
-	countries.processed.est <- country.codes.processed[idx.countries.processed[1:meta$nr_countries_estimation]]
+	countries.processed.est <- country.codes.processed[idx.countries.processed[1:ncountries.est]]
 
 	if(length(replaced.processed.est)+length(added.processed.est)+length(countries.processed.est)> 0) {
 		cat('\nCountries', paste(c(replaced.processed.est, added.processed.est, 
@@ -190,9 +189,9 @@ set.wpp.extra <- function(meta, countries=NULL, my.tfr.file=NULL, verbose=FALSE)
 					   setdiff(tfrs$added, added.processed.est),
 					   setdiff(countries, countries.processed.est))
 	if(length(include.codes) > 0) {
-		include <- is.element(tfrs$tfr_data$country_code, include.codes)
-		locations <- read.UNlocations(tfrs$tfr_data, wpp.year=meta$wpp.year, verbose=verbose)
-		TFRmatrix.regions <- get.TFRmatrix.and.regions(tfrs$tfr_data[include,], locations$loc_data, 
+		include <- is.element(tfrs$data$country_code, include.codes)
+		locations <- read.UNlocations(tfrs$data, wpp.year=meta$wpp.year, package=package, verbose=verbose)
+		TFRmatrix.regions <- get.TFRmatrix.and.regions(tfrs$data[include,], locations$loc_data, 
 												start.year=meta$start.year, 
 												present.year=meta$present.year,
 												verbose=verbose)
@@ -205,3 +204,10 @@ set.wpp.extra <- function(meta, countries=NULL, my.tfr.file=NULL, verbose=FALSE)
 	}
 	return(NULL)
 }
+
+set.wpp.extra <- function(meta, countries=NULL, my.tfr.file=NULL, verbose=FALSE) {
+	#'countries' is a vector of country or region codes 
+	tfrs <- read.UNtfr(wpp.year=meta$wpp.year, my.tfr.file=my.tfr.file, 
+							present.year=meta$present.year, verbose=verbose)
+	return(.extra.matrix.regions(tfrs, countries, meta, verbose=verbose))
+}
